const RESERVED_WORDS = [
  'SELECT',
  'UPDATE',
  'DELETE',
  'INSERT',
  'DROP',
  'UNION',
  'AND',
  'OR',
  'WHERE',
  'FROM',
  'JOIN',
  'TABLE',
  'INTO',
  'VALUES',
  'SET',
  'CREATE',
  'ALTER',
  'INDEX',
  'VIEW',
  'TRIGGER',
  'PROCEDURE',
  'FUNCTION',
  'DATABASE',
  'COLUMN',
  'KEY',
  'PRIMARY',
  'FOREIGN',
  'REFERENCES',
  'GROUP',
  'ORDER',
  'BY',
  'HAVING',
  'LIMIT',
  'OFFSET',
  'EXEC',
  'DECLARE',
  'CAST',
  'CONVERT',
];

/**
 * Array of regular expressions used to detect SQL injection patterns.
 *
 * These patterns are commonly used in SQL queries and can indicate potential SQL injection attacks.
 * Each regular expression represents a specific SQL statement or operation that should be monitored.
 *
 * @autor Andres Gutierrez Velez
 * @since 21/06/2024
 * @contact sr.willardkraft@gmail.com
 */
const SQL_PATTERNS = [
  /\bSELECT\b\s+\*\s+\bFROM\b\s+\w+/i,
  /\bINSERT\b\s+\bINTO\b\s+\w+\s+\(.*?\)\s+\bVALUES\b\s+\(.*?\)/i,
  /\bUPDATE\b\s+\w+\s+\bSET\b\s+\w+\s*=\s*.+?\s+(\bWHERE\b\s+.+)?/i,
  /\bDELETE\b\s+\bFROM\b\s+\w+(\s+\bWHERE\b\s+.+)?/i,
  /\bCREATE\b\s+\bTABLE\b\s+\w+\s*\(.*?\)/i,
  /\bDROP\b\s+\bTABLE\b\s+\w+/i,
  /\bALTER\b\s+\bTABLE\b\s+\w+\s+.*?/i,
  /\bUNION\b\s+\bSELECT\b\s+.+?\s+\bFROM\b\s+.+?/i,
  /\bJOIN\b\s+\w+\s+\bON\b\s+.+?/i,
  /\bCREATE\b\s+\bDATABASE\b\s+\w+/i,
  /\bCREATE\b\s+\bVIEW\b\s+\w+\s+\bAS\b\s+\bSELECT\b\s+.+?/i,
  /\bCREATE\b\s+\bINDEX\b\s+\w+\s+\bON\b\s+\w+\s*\(.*?\)/i,
  /\bCREATE\b\s+\bTRIGGER\b\s+\w+\s+.+?/i,
  /\bCREATE\b\s+\bPROCEDURE\b\s+\w+\s*\(.*?\)\s+.+?/i,
  /\bCREATE\b\s+\bFUNCTION\b\s+\w+\s*\(.*?\)\s+.+?/i,
  /\bALTER\b\s+\bDATABASE\b\s+\w+/i,
  /\bALTER\b\s+\bVIEW\b\s+\w+\s+\bAS\b\s+\bSELECT\b\s+.+?/i,
  /\bALTER\b\s+\bINDEX\b\s+\w+/i,
  /\bALTER\b\s+\bTRIGGER\b\s+\w+/i,
  /\bALTER\b\s+\bPROCEDURE\b\s+\w+/i,
  /\bALTER\b\s+\bFUNCTION\b\s+\w+/i,
  /\bDROP\b\s+\bDATABASE\b\s+\w+/i,
  /\bDROP\b\s+\bVIEW\b\s+\w+/i,
  /\bDROP\b\s+\bINDEX\b\s+\w+/i,
  /\bDROP\b\s+\bTRIGGER\b\s+\w+/i,
  /\bDROP\b\s+\bPROCEDURE\b\s+\w+/i,
  /\bDROP\b\s+\bFUNCTION\b\s+\w+/i,
  /\b--\b/i,
  /\b;\b/i,
  /(\bEXEC\b|\bDECLARE\b|\bCAST\b|\bCONVERT\b)/i,
];

/**
 * Checks if a given string contains a potentially dangerous SQL statement.
 *
 * @param input - The string to check.
 * @returns `true` if a potentially dangerous SQL statement is detected, otherwise `false`.
 *
 * @autor Andres Gutierrez Velez
 * @since 21/06/2024
 * @contact sr.willardkraft@gmail.com
 */
export function isSQLSentence(input: string): boolean {
  const upperInput = input.toUpperCase().trim();
  const words = upperInput.split(/\s+/);
  const reservedWordCount = words.filter((word) =>
    RESERVED_WORDS.includes(word),
  ).length;

  if (reservedWordCount > 1) {
    return true;
  }

  return SQL_PATTERNS.some((pattern) => pattern.test(upperInput));
}

/**
 * Detects if there is a potential SQL injection in a given string.
 *
 * @param input - The string to analyze.
 * @returns `true` if a potential SQL injection is detected, otherwise `false`.
 *
 * @remarks
 * This function uses a regular expression to look for common SQL injection patterns in the input string.
 * Patterns include SQL keywords like SELECT, UPDATE, DELETE, INSERT, DROP, UNION, AND, OR, as well as two or more digit numbers and the pattern '; --'.
 *
 * @example
 * ```typescript
 * const input = "SELECT * FROM users WHERE id = 1; --";
 * const isSQLInjection = detectSQLInjection(input);
 * console.log(isSQLInjection);  // Output: true
 * ```
 *
 * @autor Andres Gutierrez Velez
 * @since 21/06/2024
 * @contact sr.willardkraft@gmail.com
 */
export function detectSQLInjection(input: string): boolean {
  const sqlInjectionPattern =
    /(\b(SELECT|UPDATE|DELETE|INSERT|DROP|UNION|AND|OR|EXEC|DECLARE|CAST|CONVERT|--)|\b\d{2,}\b|['"];\s*--)/i;
  return sqlInjectionPattern.test(input);
}
